@@ -27,7 +27,7 @@
  * list of people on the gedit Team.
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-commands-view.c,v 1.5 2006/02/24 13:22:58 pborelli Exp $
+ * $Id: gedit-commands-view.c,v 1.6 2006/05/29 10:14:28 paolo Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -43,7 +43,7 @@
 
 
 void
-gedit_cmd_view_show_toolbar (GtkAction   *action,
+_gedit_cmd_view_show_toolbar (GtkAction   *action,
 			     GeditWindow *window)
 {
 	gboolean visible;
@@ -59,7 +59,7 @@ gedit_cmd_view_show_toolbar (GtkAction   *action,
 }
 
 void
-gedit_cmd_view_show_statusbar (GtkAction   *action,
+_gedit_cmd_view_show_statusbar (GtkAction   *action,
 			       GeditWindow *window)
 {
 	gboolean visible;
@@ -75,7 +75,7 @@ gedit_cmd_view_show_statusbar (GtkAction   *action,
 }
 
 void
-gedit_cmd_view_show_side_pane (GtkAction   *action,
+_gedit_cmd_view_show_side_pane (GtkAction   *action,
 			       GeditWindow *window)
 {
 	gboolean visible;
@@ -99,7 +99,7 @@ gedit_cmd_view_show_side_pane (GtkAction   *action,
 }
 
 void
-gedit_cmd_view_show_bottom_pane (GtkAction   *action,
+_gedit_cmd_view_show_bottom_pane (GtkAction   *action,
 				 GeditWindow *window)
 {
 	gboolean visible;
