@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-view.c,v 1.71 2006/08/07 16:42:40 pborelli Exp $ 
+ * $Id: gedit-view.c,v 1.72 2006/08/09 15:17:37 pborelli Exp $ 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -1561,13 +1561,12 @@ ensure_search_window (GeditView *view)
 	gtk_entry_set_completion (GTK_ENTRY (view->priv->search_entry), 
 				  completion);
 	g_object_unref (completion);
-	
+
 	gtk_widget_realize (view->priv->search_entry);
-	
+
 	view->priv->tooltips = gtk_tooltips_new ();
-	g_object_ref (G_OBJECT (view->priv->tooltips));
-	gtk_object_sink (GTK_OBJECT (view->priv->tooltips));
-	
+	g_object_ref_sink (view->priv->tooltips);
+
 	customize_for_search_mode (view);	
 }
 
