@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-window.h,v 1.6 2006/05/31 15:40:12 paolo Exp $
+ * $Id: gedit-window.h,v 1.7 2006/08/07 16:42:41 pborelli Exp $
  */
 
 #ifndef __GEDIT_WINDOW_H__
@@ -166,7 +166,14 @@ G_CONST_RETURN gchar
 
 void		 _gedit_window_set_saving_session_state	(GeditWindow         *window,
 							 gboolean             saving_session);
-							 
+
+/* these are in gedit-window because of screen safety */
+void		 _gedit_recent_add			(GeditWindow	     *window,
+							 const gchar         *uri,
+							 const gchar         *mime);
+void		 _gedit_recent_remove			(GeditWindow         *window,
+							 const gchar         *uri);
+
 G_END_DECLS
 
 #endif  /* __GEDIT_WINDOW_H__  */
