@@ -70,20 +70,12 @@ struct _FileBrowserNode
 	GdkPixbuf *emblem;
 
 	FileBrowserNode *parent;
-	guint pos;
+	gint pos;
 };
 
 struct _FileBrowserNodeDir 
 {
-	GnomeVFSURI *uri;
-	gchar *mime_type;
-	guint flags;
-	gchar *name;
-
-	GdkPixbuf *icon;
-	GdkPixbuf *emblem;
-
-	FileBrowserNode *parent;
+	FileBrowserNode node;
 	GSList *children;
 
 	GnomeVFSAsyncHandle *load_handle;
@@ -111,6 +103,9 @@ static void model_remove_node                               (GeditFileBrowserSto
 							     GtkTreePath * path,
 							     gboolean free_nodes);
 
+static void set_virtual_root_from_node                      (GeditFileBrowserStore * model,
+				                             FileBrowserNode * node);
+
 static void gedit_file_browser_store_iface_init             (GtkTreeModelIface * iface);
 static GtkTreeModelFlags gedit_file_browser_store_get_flags (GtkTreeModel * tree_model);
 static gint gedit_file_browser_store_get_n_columns          (GtkTreeModel * tree_model);
@@ -225,9 +220,7 @@ set_gvalue_from_node (GValue          *value,
 	else {
 		uri = gnome_vfs_uri_to_string (node->uri, 
 		                               GNOME_VFS_URI_HIDE_NONE);
-		g_value_set_string (value, uri);
-
-		g_free (uri);
+		g_value_take_string (value, uri);
 	}
 }
 
@@ -269,6 +262,7 @@ gedit_file_browser_store_set_property (GObject      *object,
 		case PROP_FILTER_MODE:
 			gedit_file_browser_store_set_filter_mode (obj,
 			                                          g_value_get_flags (value));
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -586,8 +580,7 @@ gedit_file_browser_store_get_value (GtkTreeModel * tree_model,
 			    gnome_vfs_uri_to_string (node->uri,
 						     GNOME_VFS_URI_HIDE_NONE);
 
-		g_value_set_string (value, uri);
-		g_free (uri);
+		g_value_take_string (value, uri);
 		break;
 	case GEDIT_FILE_BROWSER_STORE_COLUMN_NAME:
 		g_value_set_string (value, node->name);
@@ -863,12 +856,32 @@ model_node_update_visibility (GeditFileBrowserStore * model,
 	}
 }
 
+static gint
+collate_nodes (FileBrowserNode * node1, FileBrowserNode * node2)
+{
+	if (node1->name == NULL)
+		return -1;
+	else if (node2->name == NULL)
+		return 1;
+	else {
+		gchar *k1, *k2;
+		gint result;
+
+		k1 = g_utf8_collate_key_for_filename (node1->name, -1);
+		k2 = g_utf8_collate_key_for_filename (node2->name, -1);
+
+		result = strcmp (k1, k2);
+
+		g_free (k1);
+		g_free (k2);
+
+		return result;
+	}
+}
+
 static gint
 model_sort_default (FileBrowserNode * node1, FileBrowserNode * node2)
 {
-	gchar *case1;
-	gchar *case2;
-	gint result;
 	gint f1;
 	gint f2;
 
@@ -903,21 +916,7 @@ model_sort_default (FileBrowserNode * node1, FileBrowserNode * node2)
 			return 1;
 	}
 
-	if (node1->name == NULL)
-		return -1;
-	else if (node2->name == NULL)
-		return 1;
-	else {
-		case1 = g_utf8_casefold (node1->name, -1);
-		case2 = g_utf8_casefold (node2->name, -1);
-
-		result = g_utf8_collate (case1, case2);
-
-		g_free (case1);
-		g_free (case2);
-
-		return result;
-	}
+	return collate_nodes (node1, node2);
 }
 
 static void
@@ -1021,7 +1020,7 @@ model_resort_node (GeditFileBrowserStore * model, FileBrowserNode * node)
 		dir->children = g_slist_sort (dir->children,
 					      (GCompareFunc) (model->priv->
 							      sort_func));
-		neworder = g_new (gint, g_slist_length (dir->children));
+		neworder = g_new (gint, pos);
 		pos = 0;
 
 		/* Store the new positions */
@@ -1316,7 +1315,9 @@ model_remove_node (GeditFileBrowserStore * model, FileBrowserNode * node,
 
 	model_remove_node_children (model, node, path, free_nodes);
 
-	if (model_node_visibility (model, node))
+	/* Only delete if the node is visible in the tree (but only when it's
+	   not the virtual root) */
+	if (model_node_visibility (model, node) && node != model->priv->virtual_root)
 		gtk_tree_model_row_deleted (GTK_TREE_MODEL (model), path);
 
 	if (free_path)
@@ -1331,12 +1332,17 @@ model_remove_node (GeditFileBrowserStore * model, FileBrowserNode * node,
 			    g_slist_remove (FILE_BROWSER_NODE_DIR
 					    (node->parent)->children,
 					    node);
-
-		file_browser_node_free (model, node);
 	}
-
-	if (parent && model_node_visibility (model, parent))
+	
+	/* If this is the virtual root, than set the parent as the virtual root */
+	if (node == model->priv->virtual_root)
+		set_virtual_root_from_node (model, parent);
+	else if (parent && model_node_visibility (model, parent) && !(free_nodes && NODE_IS_DUMMY(node)))
 		model_check_dummy (model, parent);
+
+	/* Now free the node if necessary */
+	if (free_nodes)
+		file_browser_node_free (model, node);
 }
 
 /**
@@ -1672,7 +1678,8 @@ file_browser_node_set_from_info (GeditFileBrowserStore * model,
 
 	if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
 		node->flags |= GEDIT_FILE_BROWSER_STORE_FLAG_IS_DIRECTORY;
-	else if (gnome_vfs_mime_type_get_equivalence (node->mime_type,
+	else if (node->mime_type && 
+	         gnome_vfs_mime_type_get_equivalence (node->mime_type,
 						      "text/plain") !=
 		 GNOME_VFS_MIME_UNRELATED)
 		node->flags |= GEDIT_FILE_BROWSER_STORE_FLAG_IS_TEXT;
@@ -1798,8 +1805,6 @@ on_directory_monitor_event, parent);
 			model_check_dummy (model, parent);
 		}
 	} else {
-		g_warning ("Error on loading directory: %s",
-			   gnome_vfs_result_to_string (result));
 		g_signal_emit (model, 
 		               model_signals[ERROR], 
 		               0,
@@ -1843,17 +1848,17 @@ static GList *
 get_parent_uris (GeditFileBrowserStore * model, GnomeVFSURI * uri)
 {
 	GList *result = NULL;
-	GnomeVFSURI *parent;
 
 	result = g_list_prepend (result, gnome_vfs_uri_ref (uri));
 
 	while (gnome_vfs_uri_has_parent (uri)) {
-		parent = gnome_vfs_uri_get_parent (uri);
+		uri = gnome_vfs_uri_get_parent (uri);
 
-		if (gnome_vfs_uri_equal (parent, model->priv->root->uri))
+		if (gnome_vfs_uri_equal (uri, model->priv->root->uri)) {
+			gnome_vfs_uri_unref (uri);
 			break;
+		}
 
-		uri = gnome_vfs_uri_ref (parent);
 		result = g_list_prepend (result, uri);
 	}
 
@@ -1917,7 +1922,7 @@ model_fill (GeditFileBrowserStore * model, FileBrowserNode * node,
 	}
 
 	if (free_path)
-		g_free (path);
+		gtk_tree_path_free (path);
 }
 
 static void
@@ -1984,9 +1989,6 @@ set_virtual_root_from_node (GeditFileBrowserStore * model,
 					file_browser_node_unload (model,
 								  check,
 								  FALSE);
-
-					model_create_dummy_node (model,
-								 check);
 				}
 			} else if (check != prev) {
 				/* Only free when the node is not in the chain */
@@ -2008,24 +2010,20 @@ set_virtual_root_from_node (GeditFileBrowserStore * model,
 	for (item = FILE_BROWSER_NODE_DIR (node)->children; item;
 	     item = item->next) {
 		check = (FileBrowserNode *) (item->data);
-
-		if (NODE_IS_DIR (check)) {
+			
+		if (NODE_IS_DIR (check)) {		
 			for (copy =
 			     FILE_BROWSER_NODE_DIR (check)->children; copy;
 			     copy = copy->next) {
 				file_browser_node_free_children (model,
 								 (FileBrowserNode
 								  *)
-								 (item->
+								 (copy->
 								  data));
 				file_browser_node_unload (model,
 							  (FileBrowserNode
-							   *) (item->data),
+							   *) (copy->data),
 							  FALSE);
-
-				model_create_dummy_node (model,
-							 (FileBrowserNode
-							  *) (item->data));
 			}
 		} else if (NODE_IS_DUMMY (check)) {
 			check->flags |=
@@ -2051,7 +2049,6 @@ set_virtual_root_from_uri (GeditFileBrowserStore * model,
 	GList *item;
 	FileBrowserNode *node;
 	FileBrowserNode *parent;
-	FileBrowserNode *dummy;
 	GnomeVFSURI *check;
 	gboolean created = FALSE;
 	GnomeVFSFileInfo *info;
@@ -2086,10 +2083,6 @@ set_virtual_root_from_uri (GeditFileBrowserStore * model,
 							 info);
 			gnome_vfs_file_info_unref (info);
 
-			dummy = model_create_dummy_node (model, node);
-			dummy->flags |=
-			    GEDIT_FILE_BROWSER_STORE_FLAG_IS_HIDDEN;
-
 			model_add_node (model, node, parent);
 			created = TRUE;
 		}
@@ -2283,7 +2276,7 @@ gedit_file_browser_store_set_virtual_root (GeditFileBrowserStore * model,
 }
 
 GeditFileBrowserStoreResult
-    gedit_file_browser_store_set_virtual_root_from_string
+gedit_file_browser_store_set_virtual_root_from_string
     (GeditFileBrowserStore * model, gchar const *root) {
 	GnomeVFSURI *uri = gnome_vfs_uri_new (root);
 	gchar *str, *str1;
@@ -2415,7 +2408,7 @@ GeditFileBrowserStoreResult
 gedit_file_browser_store_set_root_and_virtual_root (GeditFileBrowserStore *
 						    model,
 						    gchar const *root,
-						    gchar const *virtual)
+						    gchar const *virtual_root)
 {
 	GnomeVFSURI *uri = NULL;
 	GnomeVFSURI *vuri = NULL;
@@ -2442,14 +2435,14 @@ gedit_file_browser_store_set_root_and_virtual_root (GeditFileBrowserStore *
 	if (root != NULL && model->priv->root != NULL) {
 		equal = gnome_vfs_uri_equal (uri, model->priv->root->uri);
 
-		if (equal && virtual == NULL) {
+		if (equal && virtual_root == NULL) {
 			gnome_vfs_uri_unref (uri);
 			return GEDIT_FILE_BROWSER_STORE_RESULT_NO_CHANGE;
 		}
 	}
 
-	if (virtual) {
-		vuri = gnome_vfs_uri_new (virtual);
+	if (virtual_root) {
+		vuri = gnome_vfs_uri_new (virtual_root);
 
 		if (equal && model->priv->virtual_root &&
 		    gnome_vfs_uri_equal (vuri,
@@ -2481,10 +2474,10 @@ gedit_file_browser_store_set_root_and_virtual_root (GeditFileBrowserStore *
 		
 		g_object_notify (G_OBJECT (model), "root");
 
-		if (virtual != NULL)
+		if (virtual_root != NULL)
 			return
 			    gedit_file_browser_store_set_virtual_root_from_string
-			    (model, virtual);
+			    (model, virtual_root);
 		else
 			set_virtual_root_from_node (model,
 						    model->priv->root);
@@ -2792,7 +2785,7 @@ gedit_file_browser_store_new_file (GeditFileBrowserStore * model,
 	g_return_val_if_fail (iter != NULL, FALSE);
 
 	parent_node = FILE_BROWSER_NODE_DIR (parent->user_data);
-	uri = unique_new_name (parent_node->uri, _("file"));
+	uri = unique_new_name (((FileBrowserNode *) parent_node)->uri, _("file"));
 
 	ret = gnome_vfs_create_uri (&handle, uri,
 				    GNOME_VFS_OPEN_NONE |
@@ -2841,7 +2834,7 @@ gedit_file_browser_store_new_directory (GeditFileBrowserStore * model,
 	g_return_val_if_fail (iter != NULL, FALSE);
 
 	parent_node = FILE_BROWSER_NODE_DIR (parent->user_data);
-	uri = unique_new_name (parent_node->uri, _("directory"));
+	uri = unique_new_name (((FileBrowserNode *) parent_node)->uri, _("directory"));
 
 	ret = gnome_vfs_make_directory_for_uri (uri, 0755);
 
